import React, { useEffect, useState } from "react";
import { DataGrid, GridColDef, GridCellEditStopParams } from "@mui/x-data-grid";
import { DewarsService } from "../../openapi";

interface SampleSpreadsheetProps {
    dewarId: number; // Selected Dewar's ID
}

const SampleSpreadsheet: React.FC<SampleSpreadsheetProps> = ({ dewarId }) => {
    const [rows, setRows] = useState<any[]>([]); // Rows for the grid
    const [columns, setColumns] = useState<GridColDef[]>([]); // Columns for the grid

    // Fetch rows and columns related to the given Dewar ID
    useEffect(() => {
        if (!dewarId) {
            console.error("Invalid or missing Dewar ID");
            return;
        }

        const fetchSamples = async () => {
            try {
                const response = await DewarsService.getDewarSamplesDewarsDewarsDewarIdSamplesGet(dewarId);
                console.log("Response from backend:", response);
                const dewarData = response; // Assume response is already in correct structure

                // Transform pucks and samples into rows for the grid
                const allRows: any[] = [];
                dewarData.pucks.forEach((puck: any) => {
                    puck.samples.forEach((sample: any) => {
                        const {
                            directory,
                            oscillation,
                            exposure,
                            totalrange,
                            transmission,
                            dose,
                        } = sample.data_collection_parameters || {};

                        allRows.push({
                            id: sample.id,
                            puckId: puck.id,
                            puckName: puck.puck_name,
                            puckType: puck.puck_type,
                            dewarName: dewarData.dewar_name,
                            position: sample.position,
                            crystalName: sample.sample_name,
                            proteinName: sample.proteinname,
                            priority: sample.priority,
                            comments: sample.comments,
                            directory: directory || null,
                            oscillation: oscillation || null,
                            exposure: exposure || null,
                            totalRange: totalrange || null,
                            transmission: transmission || null,
                            dose: dose || null,
                            dataCollectionParameters: sample.data_collection_parameters || {}, // Ensure this is never undefined
                        });
                    });
                });
                console.log("All rows for DataGrid:", allRows);
                setRows(allRows);

                // Define columns for the grid
                setColumns([
                    { field: "dewarName", headerName: "Dewar Name", width: 150, editable: false },
                    { field: "puckName", headerName: "Puck Name", width: 150 },
                    { field: "puckType", headerName: "Puck Type", width: 150 },
                    { field: "crystalName", headerName: "Crystal Name", width: 200, editable: true },
                    { field: "proteinName", headerName: "Protein Name", width: 200, editable: true },
                    { field: "position", headerName: "Position", width: 100, editable: true, type: "number" },
                    { field: "priority", headerName: "Priority", width: 100, editable: true, type: "number" },
                    { field: "comments", headerName: "Comments", width: 300, editable: true },
                    { field: "directory", headerName: "Directory", width: 200 },
                    { field: "oscillation", headerName: "Oscillation", width: 150, editable: true, type: "number" },
                    { field: "exposure", headerName: "Exposure", width: 150, editable: true, type: "number" },
                    { field: "totalRange", headerName: "Total Range", width: 150, editable: true, type: "number" },
                    { field: "transmission", headerName: "Transmission", width: 150, editable: true, type: "number" },
                    { field: "dose", headerName: "Dose", width: 150, editable: true, type: "number" },
                    {
                        field: "dataCollectionParameters",
                        headerName: "Data Collection Parameters (Raw)",
                        width: 300,
                        valueGetter: (params) =>
                            params.row?.dataCollectionParameters
                                ? JSON.stringify(params.row.dataCollectionParameters)
                                : "N/A", // Fallback if undefined
                        editable: false,
                    },
                ]);
            } catch (error) {
                console.error("Error fetching dewar samples:", error);
            }
        };

        fetchSamples();
    }, [dewarId]);

    // Handle cell editing to persist changes to the backend
    const handleCellEditStop = async (params: GridCellEditStopParams) => {
        const { id, field, value } = params;
        if (!id || !field || value === undefined) {
            console.error("Invalid edit inputs");
            return;
        }

        try {
            // Call the update_sample API endpoint
            await DewarsService.updateSampleSampleIdPut(id as number, {
                [field]: value,
            });
            console.log("Sample updated successfully");
        } catch (error) {
            console.error(`Error updating sample (id: ${id}):`, error);
        }
    };

    return (
        <div style={{ height: 500, width: "100%" }}>
            <DataGrid
                rows={rows}
                columns={columns}
                pageSize={10}
                onCellEditStop={handleCellEditStop}
            />
        </div>
    );
};

export default SampleSpreadsheet;